package XMLTest;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLHandler {

    //A @data tartalma kerül bele az XMLbe
    private ArrayList<Book> data = null;
    //Az XML dokumnetum
    private Document doc = null;
    //Az XML fájl
    private File file = null;

    public XMLHandler() {}

    public void saveXML(ArrayList<Book> _books, File _file) {
        createDocument(null);
        data = _books;
        file = _file;
        createXMLTree();
        printToFile();
    }

    public ArrayList<Book> loadXML(File _file) {
        createDocument(_file);
        data = new ArrayList<>();
        loadData();
        return data;
    }

    private void loadData() {
        doc.getDocumentElement().normalize();
        NodeList books = doc.getElementsByTagName("Book");
        for (int i = 0; i < books.getLength(); i++) {
            if(books.item(i).getNodeType() == Node.ELEMENT_NODE){
                Element bookElement = (Element) books.item(i);
                Book book = new Book();
                book.setISBN(bookElement.getAttribute("ISBN"));
                book.setAuthor(bookElement.getAttribute("Author"));
                book.setTitle(bookElement.getAttribute("Title"));
                data.add(book);
            }
        }
    }

    /**
     * Using JAXP in implementation independent manner create a document object
     * using which we create a xml tree in memory
     */
    private void createDocument(File file) {
        //get an instance of factory
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            //get an instance of builder
            DocumentBuilder db = dbf.newDocumentBuilder();
            //create an instance of DOM
            if (file == null) {
                doc = db.newDocument();
            } else {
                doc = db.parse(file);
            }
        } catch (SAXException | IOException ex) {
            Logger.getLogger(XMLHandler.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException pce) {
            System.out.println(
                    "Error while trying to instantiate DocumentBuilder " + pce);
        }
    }

    private void createXMLTree() {
        Element rootElement = doc.createElement("Books");
        doc.appendChild(rootElement);
        for (Book book : data) {
            rootElement.appendChild(createBookElement(book));
        }
    }

    private Element createBookElement(Book book) {
        Element bookElement = doc.createElement("Book");
        bookElement.setAttribute("ISBN", book.getISBN());
        bookElement.setAttribute("Author", book.getAuthor());
        bookElement.setAttribute("Title", book.getTitle());
        return bookElement;
    }

    private void printToFile() {
        try {
            OutputFormat format = new OutputFormat(doc);
            format.setIndenting(true);
            XMLSerializer serializer = new XMLSerializer(
                    new FileOutputStream(file), format);
            serializer.serialize(doc);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
}
